--/Example 08

--/ Example of using the ORDER command to order data during data retrieval from a table
--/ Here the basic ORDER command is used to sort output data by one field only using the default ASCENDING sort order

SELECT 	OrderID, CustomerID, EmployeeID, 
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM orders
WHERE 	EmployeeID = 5
ORDER BY CustomerID

--/ Adding the DESC command after the relevant field in the ORDER BY clause reverses the order naturally

SELECT 	OrderID, CustomerID, EmployeeID, 
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM orders
WHERE 	EmployeeID = 5
ORDER BY CustomerID DESC

--/ Again just like with the WHERE clause, columns/fields used for ORDER BY do NOT have to appear in the output result set
--/ In this example, just to ensure you are paying attention, I have added the WHERE IN command using a list of numbers
--/ to specify EmployeeID

SELECT 	OrderID,  
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM orders
WHERE EmployeeID IN (4,7,8)
ORDER BY CustomerID DESC


--/ You can order by multiple fields, seperating each field with a comma, each field able to have the DESC command added 
--/if required

SELECT 	OrderID, CustomerID, EmployeeID, 
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM orders
ORDER BY CustomerID, OrderDateAsString DESC

--/ NOTE the above query does NOT do what you would expect!!
--/ The ORDER BY clause can use aliases and in this case is doing so, using the OrderDateAsString varchar alias 
--/ as a sort expression which is NOT what we want. In fact we can still refer to the original table columns
--/ even if they are not present in the result set so this change to the query does what I imagine was really required

SELECT 	OrderID, CustomerID, EmployeeID, 
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM orders
ORDER BY CustomerID, OrderDate DESC





