--/Example 09

--/ Example of using other SELECT clause arguments to restrict record/row data during data retrieval from a table
--/ The default in the SELECT statement is the expression 'ALL'

SELECT 	ALL 
	OrderID, CustomerID, EmployeeID, --/ the ALL word can be removed
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ However we can change the ALL argument to TOP which comes in two forms, both of which restrict the number of records
--/ retrieved and placed in the result set.

--/ The first form of the TOP argument is just an integer and restricts how many records are retreived to that integer
--/ i.e. in this case the TOP 9first) 5 records

SELECT 	TOP 5 
	OrderID, CustomerID, EmployeeID, --/ the ALL word can be removed
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ The second form of the TOP argument is used with the PERCENT argument to say what percentage of records should be 
--/ retrieved

SELECT 	TOP 5 PERCENT 
	OrderID, CustomerID, EmployeeID, --/ the ALL word can be removed
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ Note that in both these examples the records retrieved are ordered - and therefore selected - by the natural order of
--/ records in the source table/view etc
--/ The TOP command is more frequently used with the ORDER BY clause to find the top n records ordered by values in 
-- specific columns/fields. Here we are looking at the top 10 per cent of all orders with most recent first

SELECT 	TOP 10 PERCENT
	OrderID, CustomerID, EmployeeID, --/ the ALL word can be removed
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
ORDER BY OrderDate ASC


--/ Both forms of the TOP argument can have the WITH TIES clause added - this forces the query to return additional records
--/ matching the column/field that is being used in the ORDER clause
--/ If we run the two queries above and below, we can see that one returns 83 rows whilst the WITH TIES clause
--/ adds a further 1 row - look at the last two rows and see that they have an orderdate that 'ties'!

SELECT 	TOP 10 PERCENT WITH TIES 
	OrderID, CustomerID, EmployeeID, --/ the ALL word can be removed
	CONVERT(VARCHAR(10), OrderDate, 103) AS OrderDateAsString,
	CONVERT(VARCHAR(10), RequiredDate, 103) AS RequiredDateAsString, 
	CONVERT(VARCHAR(10), ShippedDate, 103) AS ShippedDateAsString,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
ORDER BY OrderDate ASC


--/ Another useful argument in the SELECT clause, especially when working with non-keyed data, is the DISTINCT argument
--/ which does not allow any duplicate rows in the result set

--/ As an example running the query below shows 42 rows in the result set

SELECT 	CustomerID, EmployeeID, --/ the ALL word can be removed
	ShipVia, ShipName, ShipCity
FROM 	orders
WHERE 	EmployeeID = 5

--/ Adding the DISTINCT argument restricts this further and we then get 39 rows in the result set

SELECT 	DISTINCT CustomerID, EmployeeID, --/ the DISTINCT clause is added
	ShipVia, ShipName, ShipCity
FROM 	orders
WHERE 	EmployeeID = 5
