--/ Example 9B

--/ Manipulating column data in the output set with the forms of the CASE statement

--/ The CASE statement allows the query designer to modify column data based upon given record criteria
--/ with one form, called the Simple CASE function being 

--/ CASE input_expression 
--/    WHEN when_expression THEN result_expression 
--/        [ ...n ] 
--/    [ 
--/        ELSE else_result_expression 
--/    ] 
--/ END 

--/ Using this simple CASE function on our last query

SELECT 	ALL 
	OrderID, CustomerID, EmployeeID, 
	CASE ShipCountry 
	WHEN 'France' THEN 'Europe'
	WHEN 'Switzerland' THEN 'Europe'
	WHEN 'USA' THEN 'America'
	WHEN 'Finland' THEN 'Europe'
	WHEN 'Germany' THEN 'Europe'
	WHEN 'Belgium' THEN 'Europe'
	WHEN 'Venezuela' THEN 'South America'
	ELSE 'Other'
	END AS 'Ship Area',
	ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ Note that you CANNOT USE any comparison operators using the Simple CASE function so this will not work
SELECT 	ALL 
	OrderID, CustomerID, EmployeeID, 
	CASE ShipCountry 
	WHEN LIKE '%France%' THEN 'Europe'
	WHEN <> 'Switzerland' THEN 'Europe'
	WHEN 'USA' THEN 'America'
	WHEN 'Finland' THEN 'Europe'
	WHEN 'Germany' THEN 'Europe'
	WHEN 'Belgium' THEN 'Europe'
	WHEN 'Venezuela' THEN 'South America'
	ELSE 'Other'
	END AS 'Ship Area',
	ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ However the other form of the CASE function, called the SEARCHED CASE function, allows all forms of comparison
--/ and evaluates in the order specified the 'boolean expression' returning the first one that evaluates to TRUE

SELECT 	ALL 
	OrderID, CustomerID, EmployeeID, 
	CASE 
	WHEN ShipCountry LIKE '%France%' THEN 'Europe'
	WHEN ShipCountry <> 'Switzerland' THEN 'Europe'
	ELSE 'Other'
	END AS 'Ship Area',
	ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ this latter example doesn't demonstrate the reasonable sophistication of the SEARCHED CASE function

SELECT 	OrderID, CustomerID, EmployeeID, 
	OrderDate,
	CASE 
	WHEN OrderDate > CONVERT(DateTime, '01/01/1997', 103) THEN '1997 Sales'
	WHEN OrderDate > CONVERT(DateTime, '01/01/1996', 103) THEN '1996 Sales'
	WHEN OrderDate > CONVERT(DateTime, '01/01/1995', 103) THEN '1995 Sales'
	WHEN OrderDate > CONVERT(DateTime, '01/01/1994', 103) THEN '1994 Sales'
	ELSE 'Early Days'
	END AS YearSales,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ Note the quick and dirty use of the d comparison rather than writing CONVERT ....

SELECT 	OrderID, CustomerID, EmployeeID, 
	OrderDate,
	CASE 
	WHEN OrderDate > {d '1997-01-01'} THEN '1997 Sales'
	WHEN OrderDate > {d '1996-01-01'} THEN '1996 Sales'
	WHEN OrderDate > {d '1995-01-01'} THEN '1995 Sales'
	WHEN OrderDate > {d '1994-01-01'} THEN '1994 Sales'
	ELSE 'Early Days'
	END AS YearSales,
	ShipVia, Freight, ShipName, 
	ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry
FROM 	orders
WHERE 	EmployeeID = 5

--/ The Searched CASE funtion isn't restricted to a single field comparison and any expression resulting in a boolean
--/ can be used
--/ In this example we use the IN function with another query (a SUBQUERY), this one returning a single column/field 
--/ of EmployeeID

SELECT 	OrderID, CustomerID, EmployeeID, 
	CASE 
	WHEN EmployeeID IN 
	(	SELECT EmployeeID FROM Employees
		WHERE HireDate < {d '1993-01-01'}
	) THEN 'Original Employee'
	ELSE 'New Employee'
	END AS 'Employee Type'
FROM 	orders
ORDER BY [Employee Type], OrderDate

--/ SubQueries or INNER QUERY or INNER SELECT can be used in many situations and can often be formulated as JOINS (wait...)
--/ Note that when using IN and a SubQuery, the SubQuery must return a single column/field. So the following does NOT work

SELECT 	OrderID, CustomerID, EmployeeID, 
	CASE 
	WHEN EmployeeID IN 
	(	SELECT EmployeeID, Title FROM Employees
		WHERE HireDate < {d '1993-01-01'}
	) THEN 'Original Employee'
	ELSE 'New Employee'
	END AS 'Employee Type'
FROM 	orders
ORDER BY [Employee Type], OrderDate

--/ To run a query like this, it needs changing to one of type EXISTS - although I'm not a fan of the EXISTS function
--/ The EXISTS function returns a BOOLEAN expression and is used to test the existence of rows in a result set

SELECT 	OrderID, CustomerID, EmployeeID, 
	CASE 
	WHEN EXISTS
	(SELECT * FROM Employees
	 WHERE HireDate < {d '1993-01-01'}
	 AND orders.EmployeeID = Employees.EmployeeID
	) THEN 'Original Employee'
	ELSE 'New Employee'
	END AS 'Employee Type'
FROM 	orders
ORDER BY [Employee Type], OrderDate







