--/Example 11

--/ Example of using the GROUP BY, ORDER and HAVING clauses with AGGEGRATE funtions to restrict record/row data during data retrieval from a table

--/ Naturally data retrieved with a GROUP BY clause can have the result set sorted using the ORDER clause
--/ All the following 3 queries produce the same result sets
 
SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID
ORDER BY 1 DESC

--/ OR

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID
ORDER BY TotalFreightCosts DESC

--/ OR

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID
ORDER BY SUM(freight) DESC

--/ Again, we can use a field not in the result set as the order by field AS LONG AS IT IS IN THE GROUP BY clause
--/ so the following is OK

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID, EmployeeID
ORDER BY EmployeeID DESC

--/ whereas the following query is not ok
SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID
ORDER BY EmployeeID DESC

--/ We can also use the WHERE clause to further restrict what records are used in the query

SELECT 	CustomerID, SUM(freight) AS TotalFreightCosts
FROM 	orders
WHERE 	EmployeeID = 5
GROUP BY CustomerID

--/ but what you cannot do with the WHERE clause is to filter the result set on the value from the AGGREGATE function
--/ so the following is NOT valid

SELECT 	CustomerID, SUM(freight) AS TotalFreightCosts
FROM 	orders
WHERE 	SUM(freight) > 100
GROUP BY CustomerID

--/ Instead of the WHERE clause, you have to replace it with the HAVING clause so the above query then becomes
--/ noting that the HAVING clause comes at the end of the query rather than before the GROUP BY clause

SELECT 	CustomerID, SUM(freight) AS TotalFreightCosts
FROM 	orders
GROUP BY CustomerID
HAVING 	SUM(freight) > 100

--/ there is no reason not to combine WHERE with the HAVING clauses as long as you understand the difference between them
--/ so the following is valid

SELECT 	CustomerID, SUM(freight) AS TotalFreightCosts
FROM 	orders
WHERE   EmployeeID = 5
GROUP BY CustomerID
HAVING 	SUM(freight) > 100

--/ Note that aliases cannot be used in the HAVING clause so the following is NOT valid

SELECT 	CustomerID, SUM(freight) AS TotalFreightCosts
FROM 	orders
WHERE   EmployeeID = 5
GROUP BY CustomerID
HAVING 	TotalFreightCosts > 100

--/ Finding duplicate records, what products were sold for the same amount more than once 
SELECT ProductID, UnitPrice, COUNT(*) AS OrderNumbers FROM [order details] od
GROUP BY ProductID, UnitPrice
HAVING COUNT(*) <> 1

--/ or the reverse, what records exist where there is only 1 of something
--/ What suppliers only supply one productID?

SELECT SupplierID FROM products
GROUP BY supplierid
HAVING COUNT(*) = 1

--/ What products have only been ordered less than 10 times 

SELECT ProductID, COUNT(*) AS Number_Ordered FROM [order details] od
GROUP BY productID
HAVING COUNT(*) < 10
